@@ -1,6 +1,9 @@
 package vless
 
 import (
+	"google.golang.org/protobuf/proto"
+
+	"github.com/xtls/xray-core/common/errors"
 	"github.com/xtls/xray-core/common/protocol"
 	"github.com/xtls/xray-core/common/uuid"
 )
@@ -9,7 +12,7 @@ import (
 func (a *Account) AsAccount() (protocol.Account, error) {
 	id, err := uuid.ParseString(a.Id)
 	if err != nil {
-		return nil, newError("failed to parse ID").Base(err).AtError()
+		return nil, errors.New("failed to parse ID").Base(err).AtError()
 	}
 	return &MemoryAccount{
 		ID:         protocol.NewID(id),
@@ -22,7 +25,7 @@ func (a *Account) AsAccount() (protocol.Account, error) {
 type MemoryAccount struct {
 	// ID of the account.
 	ID *protocol.ID
-	// Flow of the account. May be "xtls-rprx-direct".
+	// Flow of the account. May be "xtls-rprx-vision".
 	Flow string
 	// Encryption of the account. Used for client connections, and only accepts "none" for now.
 	Encryption string
@@ -36,3 +39,11 @@ func (a *MemoryAccount) Equals(account protocol.Account) bool {
 	}
 	return a.ID.Equals(vlessAccount.ID)
 }
+
+func (a *MemoryAccount) ToProto() proto.Message {
+	return &Account{
+		Id:         a.ID.String(),
+		Flow:       a.Flow,
+		Encryption: a.Encryption,
+	}
+}
